@@ -1,35 +1,38 @@
-#!/usr/bin/env python
-# -*- coding: utf-8 -*-
-# see license https://github.com/DerwenAI/kglab#license-and-copyright
-
 """
 Subgraph transforms for visualization, graph algorithms, etc.
+
+see license https://github.com/DerwenAI/kglab#license-and-copyright
 """
 
 import typing
 
-from icecream import ic  #  type: ignore # pylint: disable=W0611,E0401
-from tqdm import tqdm  # type: ignore # pylint: disable=E0401
-import pandas as pd  # type: ignore # pylint: disable=E0401
-import pyvis.network  # type: ignore # pylint: disable=E0401
-import networkx as nx  # type: ignore # pylint: disable=E0401
+from icecream import ic  #  type: ignore # pylint: disable=W0611
+from tqdm import tqdm  # type: ignore
+import pandas as pd  # type: ignore
+import pyvis.network  # type: ignore
+import networkx as nx  # type: ignore
+import numpy as np  # type: ignore
 
-from kglab import KnowledgeGraph
-from kglab.topo import Measure
-from kglab.pkg_types import NodeLike, RDF_Node, RDF_Triple
-from kglab.util import get_gpu_count
+from .kglab import KnowledgeGraph
+from .topo import Measure
+from .pkg_types import NodeLike, RDF_Node, RDF_Triple
+from .algebra import AlgebraMixin
+from .networks import NetAnalysisMixin
+from .util import get_gpu_count
 
 
 if get_gpu_count() > 0:
-    import cudf  # type: ignore # pylint: disable=E0401
-    import cugraph # type: ignore # pylint: disable=E0401,W0611 # lgtm[py/unused-import]
+    import cudf  # type: ignore
+    import cugraph # type: ignore # pylint: disable=W0611
 
 
 class Subgraph:
     """
-Base class for projection of an RDF graph into an *algebraic object* such as a *vector*, *matrix*, or *tensor* representation, to support integration with non-RDF graph libraries.
-In other words, this class provides means to vectorize selected portions of a graph as a [*dimension*](https://mathworld.wolfram.com/Dimension.html).
-See <https://derwen.ai/docs/kgl/concepts/#subgraph>
+Base class for projection of an RDF graph into an *algebraic object* such as a *vector*,
+ *matrix*, or *tensor* representation, to support integration with non-RDF graph libraries.
+ In other words, this class provides means to vectorize selected portions of a graph as a
+ [*dimension*](https://mathworld.wolfram.com/Dimension.html).
+ See <https://derwen.ai/docs/kgl/concepts/#subgraph>
 
 Features support several areas of use cases, including:
 
@@ -40,9 +43,12 @@ Features support several areas of use cases, including:
   * embedding (deep learning)
   * probabilistic graph inference (statistical relational learning)
 
-The base case is where a *subset* of the nodes in the source RDF graph get represented as a *vector*, in the `node_vector` member.
-This provides an efficient *index* on a constructed *dimension*, solely for the context of a specific use case.
+The base case is where a *subset* of the nodes in the source RDF graph get represented as
+a *vector*, in the `node_vector` member. This provides an efficient *index* on a constructed
+*dimension*, solely for the context of a specific use case.
     """
+    kg: KnowledgeGraph
+    nx_graph: typing.Optional[nx.DiGraph] = None
 
     def __init__ (
         self,
@@ -110,7 +116,7 @@ a unique identifier (an integer index) for the `node` in the RDF graph
         id: int,
         ) -> NodeLike:
         """
-Inverse transform from an intenger to a node in the RDF graph, using the identifier as an index into the node vector.
+Inverse transform from an integer to a node in the RDF graph, using the identifier as an index into the node vector.
 
     id:
 an integer index for the `node` in the RDF graph
@@ -140,11 +146,31 @@ text (or Python object) for the serialized node
         """
         return self.kg.n3fy(node)
 
+    def check_attributes(self):
+        """
+Check if needed attributes are set.
 
-class SubgraphMatrix (Subgraph):
+        returns:
+None
+        """
+        if self.kg is None:
+            raise ValueError(
+                """`Subgraph`'s `kg` should be initialized:
+                `kglab.Subgraph(kg)`"""
+            )
+
+        # create an empty `nx.DiGraph` if none is present
+        if self.nx_graph is None:
+            # NOTE: find a way to pass `bipartite` if needed
+            self.nx_graph = self.build_nx_graph(nx.DiGraph())  # pylint: disable=E1101
+
+
+class SubgraphMatrix (Subgraph, AlgebraMixin, NetAnalysisMixin):  # pylint: disable=W0223
     """
 Projection of a RDF graph to a [*matrix*](https://mathworld.wolfram.com/AdjacencyMatrix.html) representation.
 Typical use cases include integration with non-RDF graph libraries for *graph algorithms*.
+
+SPARQL query text needs to define a subgraph as: `subject -> object`.
     """
     _SRC_DST_MAP: typing.List[str] = ["subject", "object"]
 
@@ -164,7 +190,8 @@ projecting from an RDF graph represented by a `KnowledgeGraph` object.
 the source RDF graph
 
     sparql:
-text for a SPARQL query that yields pairs to project into the *subgraph*; by default this expects the query to return bindings for `subject` and `object` nodes in the RDF graph
+text for a SPARQL query that yields pairs to project into the *subgraph*; by default this expects the query to return
+ bindings for `subject` and `object` nodes in the RDF graph
 
     bindings:
 initial variable bindings
@@ -188,9 +215,11 @@ an optional map to override the  `subject` and `object` bindings expected in the
         show_symbols: bool = False,
         ) -> pd.DataFrame:
         """
-Factory pattern to populate a [`pandas.DataFrame`](https://pandas.pydata.org/docs/reference/frame.html) object, using transforms in this subgraph.
+Factory pattern to populate a [`pandas.DataFrame`](https://pandas.pydata.org/docs/reference/frame.html) object,
+using transforms in this subgraph.
 
-Note: this method is primarily intended for [`cuGraph`](https://docs.rapids.ai/api/cugraph/stable/) support. Loading via a `DataFrame` is required – in lieu of using the `nx.add_node()` approach.
+Note: this method is primarily intended for [`cuGraph`](https://docs.rapids.ai/api/cugraph/stable/) support.
+Loading via a `DataFrame` is required in lieu of using the `nx.add_node()` approach.
 Therefore the support for representing *bipartite* graphs is still pending.
 
     show_symbols:
@@ -200,6 +229,10 @@ optionally, include the symbolic representation for each node; defaults to `Fals
 the populated `DataFrame` object; uses the [RAPIDS `cuDF` library](https://docs.rapids.ai/api/cudf/stable/) if GPUs are enabled
         """
         col_names: typing.List[str] = [ "src", "dst", "src_sym", "dst_sym" ]
+
+        if self.sparql is None and self.kg.use_gpus is True:
+            raise ValueError("""To use GPUs is necessary to provide a SPARQL query to define a subgraph:
+                            `kglab.SubgraphMatrix(kg, sparql)` or `SubgraphTensor(...)`""")
         row_iter = self.kg.query(self.sparql, bindings=self.bindings)
 
         if not show_symbols:
@@ -223,7 +256,7 @@ the populated `DataFrame` object; uses the [RAPIDS `cuDF` library](https://docs.
                 for row in row_iter
                 ]
 
-        if self.kg.use_gpus:
+        if self.kg.use_gpus is True:
             df = cudf.DataFrame(rows_list, columns=col_names)
         else:
             df = pd.DataFrame(rows_list, columns=col_names)
@@ -250,7 +283,7 @@ flag for whether the `(subject, object)` pairs should be partitioned into *bipar
     returns:
 the populated `NetworkX` graph object; uses the [RAPIDS `cuGraph` library](https://docs.rapids.ai/api/cugraph/stable/) if GPUs are enabled
         """
-        if self.kg.use_gpus:
+        if self.kg.use_gpus is True:
             df = self.build_df()
             nx_graph.from_cudf_edgelist(df, source="src", destination="dst")
         else:
@@ -304,6 +337,14 @@ the populated  `iGraph` graph object
         ig_graph.vs["label"] = ig_graph.vs["name"] # pylint: disable=E1136,E1137
         return ig_graph
 
+    def _get_n_nodes(self):
+        """ Return number of nodes counted from the adjancency matrix"""
+        return self.to_adjacency().shape[0]
+
+    def _get_n_edges(self):
+        """ Return number of edges counted from the adjancency matrix"""
+        return int(np.sum(self.to_adjacency()))
+
 
 class SubgraphTensor (Subgraph):
     """
@@ -397,10 +438,10 @@ boolean flag to disable `tqdm` progress bar calculation and output
     returns:
 an edge list for the loaded tensor object
         """
-        edge_list: list = [
+        edge_list: list = list(
             edge_tuple
             for edge_tuple in tqdm(self.as_tensor_edges(), disable=quiet)  # pylint: disable=R1721
-            ]
+        )
 
         return edge_list
 
@@ -413,14 +454,14 @@ an edge list for the loaded tensor object
     ## to extend or create an analyst's account-specific network
     ## model.
 
-    def pyvis_style_node ( # pylint: disable=R0201
+    def pyvis_style_node (  # pylint: disable=R0201
         self,
         pyvis_graph: pyvis.network.Network,
         node_id: int,
         label: str,
         *,
         style: dict = None,
-        ) -> None :
+        ) -> None : # pylint: disable=R0201
         """
 Adds a node into a [PyVis](https://pyvis.readthedocs.io/) network, optionally with styling info.
 
